package com.caimei365.user.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.user.components.WeChatService;
import com.caimei365.user.mapper.LoginMapper;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.vo.UserLoginVo;
import com.caimei365.user.service.LoginService;
import com.caimei365.user.components.RedisService;
import com.caimei365.user.utils.JwtUtil;
import com.caimei365.user.utils.Md5Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/8
 */
@Slf4j
@Service
public class LoginServiceImpl implements LoginService {

    @Resource
    private RedisService redisService;
    @Resource
    private WeChatService weChatService;
    @Resource
    private LoginMapper loginMapper;

    /**
     * 登录(用户名，密码)
     *
     * @param mobileOrEmail 手机号或邮箱
     * @param password      密码
     * @return BaseUser
     */
    @Override
    public ResponseJson<UserLoginVo> passwordLogin(String mobileOrEmail, String password) {
        if (StringUtils.isBlank(mobileOrEmail)) {
            return ResponseJson.error("请填写账户名", null);
        }
        if (StringUtils.isBlank(password)) {
            return ResponseJson.error("请填写密码", null);
        }
        //处理比对密码
        UserLoginVo baseUser = loginMapper.getLoginUserByMobileOrEmail(mobileOrEmail);
        if (baseUser != null) {
            String md5Password = Md5Util.md5(password);
            String dbPassword = baseUser.getPassword();
            if (md5Password.equals(dbPassword)) {
                if (baseUser.getUserIdentity() == 1) {
                    // 协销登录
                    return ResponseJson.success(baseUser);
                } else {
                    // 返回登录校验结果
                    return logonVerify(baseUser);
                }
            }
        }
        return ResponseJson.error("输入的密码和账户名不匹配", null);
    }

    /**
     * 微信授权登录（小程序）
     *
     * @param code              微信授权code
     * @param encryptedData     微信加密数据
     * @param iv                加密算法的初始向量
     * @param headers           HttpHeaders
     */
    @Override
    public ResponseJson<UserLoginVo> appletsAuthorization(String code, String encryptedData, String iv, HttpHeaders headers) {
        if (StringUtils.isBlank(code)) {
            return ResponseJson.error("没有获取到微信授权code", null);
        }
        // 小程序微信授权获取登录信息
        ResponseJson<Map<String, Object>> appletsInfo = weChatService.getInfoMapByApplets(code, headers);
        if (appletsInfo.getCode() == -1) {
            return ResponseJson.error(appletsInfo.getMsg(), null);
        }
        Map<String, Object> infoData = appletsInfo.getData();
        String openId = (String) infoData.get("openId");
        String unionId = (String) infoData.get("unionId");
        String sessionKey = (String) infoData.get("sessionKey");
        try {
            if (StringUtils.isEmpty(unionId) || StringUtils.isBlank(unionId)) {
                String result = WeChatService.decrypt(encryptedData, sessionKey, iv, "UTF-8");
                log.info("解密数据>>>>>>" + result);
                Map parseMap = JSONObject.parseObject(result, Map.class);
                assert parseMap != null;
                unionId = parseMap.get("unionId").toString();
                infoData.put("unionId", unionId);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseJson.error("微信解密失败", null);
        }
        // 用户数据存入Redis，key前缀：wxInfo:applets:
        redisService.setMap("wxInfo:applets:" + unionId, infoData);
        log.info("小程序授权登录，返回unionId给前端，用户数据存入Redis，key：wxInfo:applets:" + unionId);
        // 协销授权登录
        UserLoginVo seller = loginMapper.getServiceProviderByOpenId(openId);
        if (null != seller) {
            loginMapper.updateServiceProviderUnionId(seller.getUserId(), unionId);
            String token = JwtUtil.createToken(seller.getUserId());
            seller.setToken(token);
            seller.setUnionId(unionId);
            seller.setOpenId(openId);
            return ResponseJson.success(seller);
        }
        // 运营人员授权登录
        return operationAuthLogin(openId, unionId, "mini");
    }

    /**
     * 微信公众号授权链接(www)
     *
     * @param redirectUri 用于微信授权的中间页面
     * @param mode        授权方式：1静默授权，其他手动同意授权
     */
    @Override
    public ResponseJson<String> getAuthorizationLink(String redirectUri, Integer mode) {
        String link = weChatService.getAuthorizationLink(redirectUri, mode);
        String state = UUID.randomUUID().toString();
        redisService.set(state, state, 1800L);
        link = link.replace("STATE", state);
        return ResponseJson.success(link);
    }

    /**
     * 微信公众号授权登录(www)
     * <p>
     * spi旧接口：user/authorizationLogin
     *
     * @param code              微信code
     * @param state             安全认证
     * @param mode              1:静默授权,2:用户手动授权
     * @param headers           HttpHeaders
     */
    @Override
    public ResponseJson<UserLoginVo> websiteAuthorization(String code, String state, Integer mode, HttpHeaders headers) {
        if (StringUtils.isBlank(code) || StringUtils.isBlank(state)) {
            return ResponseJson.error("参数异常", null);
        }
        String wxState = (String) redisService.get(state);
        log.info("微信code>>>" + code + "state>>>" + wxState + "----" + state + "mode>>>" + mode);
        if (wxState.equals(state)) {
            try {
                // 通过code获取微信用户信息
                Map<String, Object> map = weChatService.getInfoMapByWeb(code, "crm");
                String openId = (String) map.get("openid");
                if (mode == 1) {
                    // 静默授权
                    Integer userId = loginMapper.getUserIdByOpenId(openId, "www");
                    if (null != userId && userId > 0) {
                        UserLoginVo user = loginMapper.getLoginUserByUserId(userId);
                        // 返回登录用户
                        return logonVerify(user);
                    } else {
                        return ResponseJson.error(-4, "您的微信尚未绑定任何机构", null);
                    }
                } else {
                    // 获取access_token
                    String accessToken = weChatService.getAccessToken();
                    // 获取微信用户信息
                    Map<String, Object> infoData = weChatService.getUserInfo(accessToken, openId);
                    String unionId = (String) infoData.get("unionid");
                    // 用户数据存入Redis，key前缀：wxInfo:website:
                    redisService.setMap("wxInfo:website:" + unionId, infoData);
                    log.info("移动端授权登录，返回unionId给前端，用户数据存入Redis，key：wxInfo:website:" + unionId);
                    // 运营人员授权登录
                    return operationAuthLogin(openId, unionId, "www");
                }
            } catch (Exception e) {
                e.printStackTrace();
                return ResponseJson.error("获取微信信息异常", null);
            }
        }
        return ResponseJson.error("请从正确的途径打开链接", null);
    }

    /**
     * 运营人员授权登录
     * @param openId 微信openId
     * @param unionId 微信unionId
     * @param source 来源：www网站, mini小程序
     * @return BaseUser
     */
    private ResponseJson<UserLoginVo> operationAuthLogin(String openId, String unionId, String source) {
        UserLoginVo operation = loginMapper.getOperationUserByUnionId(unionId, source);
        if (null == operation) {
            operation = loginMapper.getOperationUserByOpenId(openId, source);
            if (null == operation) {
                operation = new UserLoginVo();
                operation.setOpenId(openId);
                operation.setUnionId(unionId);
                return ResponseJson.error("您的微信尚未绑定任何机构", operation);
            } else {
                // 表示 openId存在, unionId不存在
                loginMapper.updateOperationUnionId(operation.getUserId(), unionId);
            }
        }
        // 如果unionId存在, openId不存在
        if (StringUtils.isEmpty(operation.getOpenId()) || StringUtils.isBlank(operation.getOpenId())) {
            loginMapper.updateOperationOpenId(operation.getUserId(), openId);
        }
        operation.setOpenId(openId);
        operation.setUnionId(unionId);
        // 返回登录校验结果
        return logonVerify(operation);
    }
    /**
     * 登录校验
     * @param baseUser 用户信息
     * @return BaseUser
     */
    private ResponseJson<UserLoginVo> logonVerify(UserLoginVo baseUser) {
        // 生成token给用户
        String token = JwtUtil.createToken(baseUser.getUserId());
        // 为了过期续签，将token存入redis，并设置超时时间
        redisService.set(token, token, JwtUtil.getExpireTime());
        baseUser.setToken(token);
        if (baseUser.getStatus() == 91) {
            //机构 / 供应商
            return ResponseJson.error(-2, "您的企业账号已被冻结，请联系客服处理", null);
        }
        // 供应商
        if (baseUser.getUserIdentity() == 3) {
            if (baseUser.getStatus() == 3) {
                return ResponseJson.error(-2, "您的企业账号正在加速审核中,审核通过后即可登录", null);
            }
            if (baseUser.getStatus() == 92) {
                return ResponseJson.error(-3, "您的企业账号审核未通过", baseUser);
            }
        }
        return ResponseJson.success("登录成功", baseUser);
    }

    /**
     * 微信扫码登录
     *
     * @param code      微信code
     * @param state     安全认证
     */
    @Override
    public void qrCodeAuthorization(String code, String state) {
        Map<String, Object> map = new HashMap<>();
        // 简单验证，防止csrf攻击（跨站请求伪造攻击）
        String stateCache = (String) redisService.get("state:" + state);
        if (stateCache == null) {
            map.put("-1", "请从正确的途径打开链接");
        }
        if (StringUtils.isEmpty(code)) {
            map.put("-1", "请重新进行授权登录");
        }
        Map<String, Object> tokenMap;
        Map<String, Object> wxInfoMap = null;
        try {
            tokenMap = weChatService.getInfoMapByWeb(code, "pc");
            String access_token = (String) tokenMap.get("access_token");
            String openid = (String) tokenMap.get("openid");
            log.info("wx回调openid>>>>>" + openid);
            log.info("wx回调access_token>>>>>" + access_token);
            wxInfoMap = weChatService.getUserInfo(access_token, openid);
            log.info("wx回调openid>>>>>" + wxInfoMap.get("openid"));
            log.info("wx回调unionid>>>>>" + wxInfoMap.get("unionid"));
        } catch (Exception e) {
            map.put("-1", "获取微信用户信息失败");
        }
        redisService.setMap("error:" + state, map);
        redisService.setMap(state, wxInfoMap);
    }

    /**
     * netty长连接验证登录
     * @param state     安全认证
     * @return
     */
    @Override
    public ResponseJson nettyAuthorization(String state) {
        if (state == null) {
            return ResponseJson.error("参数异常");
        }
        Map<Object, Object> error = redisService.getEntries("error:" + state);
        if (null != error && error.size() > 0) {
            return ResponseJson.error(-1, "错误信息", error);
        }
        Map<Object, Object> wxInfoMap = redisService.getEntries(state);
        if (null == wxInfoMap || wxInfoMap.size() == 0) {
            return ResponseJson.error(-90, "redis缓存没有拿到");
        }
        redisService.remove("state");
        /*HttpSession session = request.getSession();
        session.setAttribute("wxInfoMap", wxInfoMap);*/
        String unionId = (String) wxInfoMap.get("unionid");
        log.info("pc商城unionId>>>>>>" + unionId);
        String openid = (String) wxInfoMap.get("openid");
        log.info("pc商城openid>>>>>>" + openid);
        UserLoginVo operation = loginMapper.getOperationUserByUnionId(unionId, "www");
        if (operation == null) {
            operation = loginMapper.getOperationUserByOpenId(openid, "www");
            if (operation == null) {
                return ResponseJson.error(-4, "您的微信尚未绑定任何机构", wxInfoMap);
            } else if (StringUtils.isBlank(operation.getUnionId())) {
                // 如果openId存在, unionId不存在
                loginMapper.updateOperationUnionId(operation.getUserId(), unionId);
            }
        // 如果unionId存在, openId不存在
        } else if (operation.getOpenId() == null) {
            loginMapper.updateOperationOpenId(operation.getUserId(), openid);
        }
        // 获取用户信息
        UserLoginVo user = loginMapper.getLoginUserByUserId(operation.getUserId());
        return logonVerify(user);
    }
}
