package com.caimei365.user.service.impl;

import com.caimei365.user.mapper.BaseMapper;
import com.caimei365.user.mapper.LoginMapper;
import com.caimei365.user.mapper.RegisterMapper;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.model.dto.ClubRegisterDto;
import com.caimei365.user.model.dto.ClubUpgradeDto;
import com.caimei365.user.model.po.ClubPo;
import com.caimei365.user.model.po.OperationPo;
import com.caimei365.user.model.po.UserPo;
import com.caimei365.user.service.RegisterService;
import com.caimei365.user.components.RedisService;
import com.caimei365.user.utils.AliyunSmsUtil;
import com.caimei365.user.utils.Md5Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/8
 */
@Slf4j
@Service
public class RegisterServiceImpl implements RegisterService {

    @Resource
    private RedisService redisService;
    @Resource
    private BaseMapper baseMapper;
    @Resource
    private LoginMapper loginMapper;
    @Resource
    private RegisterMapper registerMapper;

    /**
     * 普通机构入驻(注册)
     *
     * @param clubRegisterDto ClubRegisterDto{
     *                        source            注册来源: 0网站 1小程序
     *                        userName          用户名
     *                        bindMobile        企业绑定手机号
     *                        password          密码
     *                        unionId           微信unionId
     *                        nickName          微信昵称
     *                        avatarUrl         微信头像(旧：headimgurl)
     *                        }
     * @param passWordConfirm 用户确认密码
     * @param smsCode         短信验证码(旧：activationCode)
     * @param isAgreed        是否同意勾选同意协议，1是，其他否
     * @param headers         HttpHeaders
     * @return ClubPo
     */
    @Override
    public ResponseJson<ClubPo> clubRegister(ClubRegisterDto clubRegisterDto, String passWordConfirm, String smsCode, Integer isAgreed, HttpHeaders headers) {
        // 打印IP
        String ip = headers.getFirst("X-CLIENT-IP");
        log.info("机构注册 X-CLIENT-IP : " + ip);
        // 参数校验
        if (StringUtils.isBlank(clubRegisterDto.getUserName()) || StringUtils.isBlank(clubRegisterDto.getBindMobile())
            || StringUtils.isBlank(clubRegisterDto.getPassword()) || StringUtils.isBlank(passWordConfirm) ||
            StringUtils.isBlank(smsCode)) {
            return ResponseJson.error("参数异常", null);
        }
        if (!clubRegisterDto.getPassword().equals(passWordConfirm)) {
            return ResponseJson.error("输入的密码不一致", null);
        }
        String redisSmsCode = (String) redisService.get("code:" + clubRegisterDto.getBindMobile());
        if (!redisSmsCode.equals(smsCode)) {
            return ResponseJson.error("手机验证码错误", null);
        }
        if (1 != isAgreed) {
            return ResponseJson.error("请勾选同意协议", null);
        }
        // 查找用户表是否存在
        Integer dbUserId = baseMapper.getUserIdByMobile(clubRegisterDto.getBindMobile());
        if (null != dbUserId && dbUserId > 0) {
            return ResponseJson.error("该手机号已被使用", null);
        }
        // 查找运营人员表是否存在
        Integer dbOperationId = baseMapper.getOperationIdByMobile(clubRegisterDto.getBindMobile());
        if (null != dbOperationId && dbOperationId > 0) {
            return ResponseJson.error("您已是机构运营人员，无需再注册机构", null);
        }
        /*
            组装用户数据 user
         */
        UserPo user = new UserPo();
        // 设置日期时间格式
        Date date = new Date();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String current = dateFormat.format(date);
        // 注册时间
        user.setRegisterTime(current);
        // 注册IP
        user.setRegisterIp(ip);
        // 注册来源: 0网站 1小程序
        user.setSource(clubRegisterDto.getSource());
        // 用户类型，供应商1，会员机构3,普通机构4
        user.setRegisterUserTypeId(4);
        // 组织名称
        user.setName(clubRegisterDto.getUserName());
        user.setUserName(clubRegisterDto.getUserName());
        // 绑定手机号
        user.setBindMobile(clubRegisterDto.getBindMobile());
        // 用户身份: 1协销 2会员机构 3供应商 4普通机构
        user.setUserIdentity(4);
        // 用户权限: 2会员机构 3供应商 4协销 5普通机构
        user.setUserPermission(5);
        // 设置机构上线
        user.setClubStatus(90);
        // 设置密码
        user.setPassword(Md5Util.md5(clubRegisterDto.getPassword()));
        // 同意协议
        user.setAgreeFlag(isAgreed);
        // 用户状态，1正常，0冻结
        user.setValidFlag(1);
        // 协销Id，默认采美官方
        user.setServiceProviderId(1342);
        // 协销状态，已上线
        user.setServiceProviderStatus(90);
        /*
            保存数据库 user
         */
        int insertFlag = registerMapper.insertClubUser(user);
        log.info("插入数据库User表，获得userId:"+user.getUserId());
        if (insertFlag < 1) {
            throw new RuntimeException("插入数据库异常user:" + user.toString());
        }
        /*
            组装机构数据
         */
        ClubPo club = new ClubPo();
        // 机构名称
        club.setName(user.getUserName());
        // 机构简称
        club.setSName(user.getUserName());
        // 联系手机
        club.setContractMobile(user.getBindMobile());
        // 联系人
        club.setLinkMan(user.getUserName());
        // 用户Id
        club.setUserId(user.getUserId());
        // 协销Id(spId)
        club.setServiceProviderId(1342);
        // 注册时间
        club.setAddTime(current);
        // 状态设置上线
        club.setStatus(90);
        /*
            保存数据库 club
         */
        int insertClubFlag = registerMapper.insertClub(club);
        log.info("插入数据库club表，获得clubId:"+club.getClubId());
        if (insertClubFlag < 1) {
            throw new RuntimeException("插入数据库异常club:" + club.toString());
        }
        // user更新clubId
        user.setClubId(club.getClubId());
        registerMapper.updateUserClubId(user.getUserId(), club.getClubId());
        // 注册成功短信
        boolean smsFlag = AliyunSmsUtil.sendSms(clubRegisterDto.getBindMobile(), 7, "{name:"+ clubRegisterDto.getBindMobile() +",password:"+ clubRegisterDto.getPassword() +"}");
        if (!smsFlag) {
            // 短信发送失败重试一次
            AliyunSmsUtil.sendSms(clubRegisterDto.getBindMobile(), 7, "{name:"+ clubRegisterDto.getBindMobile() +",password:"+ clubRegisterDto.getPassword() +"}");
        }
        /*
            绑定微信，成为机构运营人员
         */
        if (StringUtils.isNotBlank(clubRegisterDto.getNickName())) {
            OperationPo operation = new OperationPo();
            // 用户Id
            operation.setUserId(user.getUserId());
            // 机构Id
            operation.setClubId(club.getClubId());
            // 微信昵称
            operation.setNickName(clubRegisterDto.getNickName());
            // 微信头像
            operation.setAvatarUrl(clubRegisterDto.getAvatarUrl());
            // 用户类型，1:机构类型，2供应商类型
            operation.setUserType(1);
            // 运营人员手机号
            operation.setMobile(clubRegisterDto.getBindMobile());
            // 联系人
            operation.setLinkName(clubRegisterDto.getUserName());
            // 运营人员状态：1未绑定，2已绑定
            operation.setStatus(2);
            // 微信unionId
            operation.setUnionId(clubRegisterDto.getUnionId());
            Map<Object, Object> infoData = redisService.getEntries("wxInfo:applets:" + clubRegisterDto.getUnionId());
            String openId = (String) infoData.get("openId");
            // 微信openId
            operation.setOpenId(openId);
            // 添加时间
            operation.setAddTime(date);
            // 更新时间
            operation.setUpdateTime(date);
            // 绑定时间
            operation.setBindTime(date);
            // delFlag: 0 有效  其它无效
            operation.setDelFlag(0);
            /*
                保存数据库 operation
             */
            int insertOperationFlag = registerMapper.insertOperation(operation);
            log.info("插入数据库cm_mall_operation_user表，获得id:"+operation.getId());
            if (insertOperationFlag < 1) {
                log.info(operation.getUserId() + " 插入数据库异常operation:" + operation.toString());
            }
            log.info("注册普通机构,默认绑定为机构运营人员");
        }
        log.info("注册普通机构成功,手机号>>>" + clubRegisterDto.getBindMobile());
        return ResponseJson.success(club);
    }

    /**
     * 普通机构升级会员机构
     *
     * @param upgradeDto ClubUpgradeDto:{
     *                      clubId                 机构ID
     *                      userId                 用户ID
     *                      name                   机构名称
     *                      sName                  机构简称
     *                      contractEmail          邮箱（contractEmail1）
     *                      contractPhone          固定电话
     *                      linkMan                联系人（linkMan1）
     *                      provinceId             省Id
     *                      cityId                 市Id
     *                      townId                 县区Id
     *                      address                地址
     *                      shopPhoto              门头照(headpic)
     *                      businessLicense        营业执照（businessLicenseImage）
     *                      socialCreditCode       统一社会信用代码（socialCreditCode）
     *                      firstClubType          一级分类：医美=1和生美=2
     *                      secondClubType         医美的二级分类：诊所=1、门诊=2、医院=3。  生美没有二级分类
     *                      department             医美分类下的门诊和医院则需要填写科室
     *                      medicalPracticeLicense 医美分类必须上传医疗执业许可证（medicalPracticeLicenseImg）
     *                      mainProduct            主打项目（mainpro）
     *                      fax                    传真
     *                      profile                公司简介
     *                  }
     * @param headers   HttpHeaders
     * @return ClubPo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJson<ClubPo> clubUpgrade(ClubUpgradeDto upgradeDto, HttpHeaders headers) {
        // 打印IP
        String ip = headers.getFirst("X-CLIENT-IP");
        log.info("机构升级 X-CLIENT-IP : " + ip);
        // 参数校验
        if (upgradeDto.getUserId() == null || upgradeDto.getClubId() == null || StringUtils.isBlank(upgradeDto.getContractEmail()) || StringUtils.isBlank(upgradeDto.getName())
                || StringUtils.isBlank(upgradeDto.getBusinessLicense()) || upgradeDto.getTownId() == null || upgradeDto.getFirstClubType() == null) {
            return ResponseJson.error("参数异常", null);
        }
        if (upgradeDto.getFirstClubType() == 1){
            if (upgradeDto.getSecondClubType() == null || StringUtils.isBlank(upgradeDto.getDepartment()) || StringUtils.isBlank(upgradeDto.getMedicalPracticeLicense())) {
                return ResponseJson.error("医美分类下参数异常", null);
            }
        }
        Integer userIdByEmail = baseMapper.getUserIdByEmail(upgradeDto.getContractEmail());
        if (null != userIdByEmail && userIdByEmail > 0 ) {
            return ResponseJson.error("该邮箱已被使用", null);
        }
        // 更新用户数据 user
        UserPo user = new UserPo();
        user.setUserId(upgradeDto.getUserId());
        // 客户IP
        user.setRegisterIp(ip);
        // 联系邮箱
        user.setEmail(upgradeDto.getContractEmail());
        // 机构名称
        user.setName(upgradeDto.getName());
        // 待审核资料
        user.setClubStatus(1);
        // 更新机构用户
        registerMapper.updateClubUserByUpgrade(user);
        /*
            组装机构数据
         */
        ClubPo club = new ClubPo();
        // 机构Id
        club.setClubId(upgradeDto.getClubId());
        // 用户Id
        club.setUserId(upgradeDto.getUserId());
        // 机构名称
        club.setName(upgradeDto.getName());
        // 机构简称
        club.setSName(upgradeDto.getSName());
        // 联系邮箱
        club.setContractEmail(upgradeDto.getContractEmail());
        // 固定电话
        club.setContractPhone(upgradeDto.getContractPhone());
        // 联系人
        club.setLinkMan(upgradeDto.getLinkMan());
        // 地址
        club.setProvinceId(upgradeDto.getProvinceId());
        club.setCityId(upgradeDto.getCityId());
        club.setTownId(upgradeDto.getTownId());
        club.setAddress(upgradeDto.getAddress());
        // 门头照
        club.setShopPhoto(upgradeDto.getShopPhoto());
        // 营业执照
        club.setBusinessLicense(upgradeDto.getBusinessLicense());
        // 统一社会信用代码
        club.setSocialCreditCode(upgradeDto.getSocialCreditCode());
        // 分类: 1医美, 2生美
        club.setFirstClubType(upgradeDto.getFirstClubType());
        if (upgradeDto.getFirstClubType() == 1){
            // 医美二级分类: 1诊所、2门诊、3医院
            club.setSecondClubType(upgradeDto.getSecondClubType());
            // 门诊和医院则需要填写科室
            club.setDepartment(upgradeDto.getDepartment());
            // 医美分类必须上传医疗执业许可证
            club.setMedicalPracticeLicense(upgradeDto.getMedicalPracticeLicense());
        }
        // 主打项目(mainpro)
        club.setMainProduct(upgradeDto.getMainProduct());
        // 传真
        club.setFax(upgradeDto.getFax());
        // 公司简介(info)
        club.setProfile(upgradeDto.getProfile());
        /*
         * 待审核资料
         */
        club.setStatus(1);
        registerMapper.updateClubByUpgrade(club);
        return ResponseJson.success(club);
    }
}

